#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include "iolib.h"
const char *progname = "softlimit";
const char *proghelp =
	"Usage: softlimit [OPTIONS] command ...\n"
	"Run a command with soft limits set.\n\n"
	"-m N      Limit DATA, STACK, MEMLOCK, AS to N\n"
	"-d N      Limit DATA to N\n"
	"-s N      Limit STACK to N\n"
	"-l N      Limit MEMLOCK to N\n"
	"-a N      Limit AS to N\n"
	"-o N      Limit NOFILE to N\n"
	"-p N      Limit NPROC to N\n"
	"-f N      Limit FSIZE to N\n"
	"-c N      Limit CORE to N\n"
	"-r N      Limit RSS to N\n"
	"-t N      Limit CPU to N\n"
	"(N above may be \"=\" to use the hard limit.)\n";
#include "freedt.h"

void set(int resource, char *value) {
	struct rlimit lim;

	if (getrlimit(resource, &lim) < 0)
		die("failed to get limit");

	if (strcmp(value, "=") == 0) {
		lim.rlim_cur = lim.rlim_max;
	} else {
		lim.rlim_cur = atoi(value);
	}

	if (setrlimit(resource, &lim) < 0)
		die("failed to set limit");
}

int main(int argc, char **argv) {
	while (1) {
		int c = getopt(argc, argv, "+V?m:d:s:l:a:o:p:f:c:r:t:");
		if (c == -1)
			break;

		switch (c) {
		case 'm':
			set(RLIMIT_DATA, optarg);
			set(RLIMIT_STACK, optarg);
			set(RLIMIT_MEMLOCK, optarg);
			set(RLIMIT_AS, optarg);
			break;
		case 'd':
			set(RLIMIT_DATA, optarg);
			break;
		case 's':
			set(RLIMIT_STACK, optarg);
			break;
		case 'l':
			set(RLIMIT_MEMLOCK, optarg);
			break;
		case 'a':
			set(RLIMIT_AS, optarg);
			break;
		case 'o':
			set(RLIMIT_NOFILE, optarg);
			break;
		case 'p':
			set(RLIMIT_NPROC, optarg);
			break;
		case 'f':
			set(RLIMIT_FSIZE, optarg);
			break;
		case 'c':
			set(RLIMIT_CORE, optarg);
			break;
		case 'r':
			set(RLIMIT_RSS, optarg);
			break;
		case 't':
			set(RLIMIT_CPU, optarg);
			break;
		case 'V':
			version();
		default:
			help();
		}
	}

	if ((argc - optind) < 1)
		help();

	execvp(argv[optind], &argv[optind]);
	die2(argv[optind], "unable to exec");

	return 0; /* NOTREACHED */
}

